{-# OPTIONS_GHC -fno-warn-unused-binds #-}

{- |
   Module     : Text.HXT.RelaxNG.Schema

   Don't edit this module, it's generated by RelaxSchemaToXmlTree

-}

module Text.XML.HXT.RelaxNG.Schema
    ( relaxSchemaTree, relaxSchemaArrow )
where

import Text.XML.HXT.DOM.TypeDefs
import Text.XML.HXT.DOM.XmlNode (mkRoot, mkElement, mkAttr, mkText)

import Control.Arrow.ListArrows

relaxSchemaArrow :: ArrowList a => a b XmlTree
relaxSchemaArrow = constA relaxSchemaTree

relaxSchemaTree :: XmlTree
relaxSchemaTree =
  let
  ns1 = "http://relaxng.org/ns/structure/1.0"
  qn1        = mkName   "RelaxContext-xml"
  mkelem_qn1 = mkElement qn1
  mkattr_qn1 = mkAttr    qn1
  qn2        = mkName   "RelaxContextBaseURI"
  mkelem_qn2 = mkElement qn2
  mkattr_qn2 = mkAttr    qn2
  qn3        = mkName   "RelaxContextDefault"
  mkelem_qn3 = mkElement qn3
  mkattr_qn3 = mkAttr    qn3
  qn4        = mkNsName "anyName" ns1
  mkelem_qn4 = mkElement qn4
  mkattr_qn4 = mkAttr    qn4
  qn5        = mkNsName "attribute" ns1
  mkelem_qn5 = mkElement qn5
  mkattr_qn5 = mkAttr    qn5
  qn6        = mkNsName "choice" ns1
  mkelem_qn6 = mkElement qn6
  mkattr_qn6 = mkAttr    qn6
  qn7        = mkNsName "data" ns1
  mkelem_qn7 = mkElement qn7
  mkattr_qn7 = mkAttr    qn7
  qn8        = mkName   "datatypeLibrary"
  mkelem_qn8 = mkElement qn8
  mkattr_qn8 = mkAttr    qn8
  qn9        = mkNsName "define" ns1
  mkelem_qn9 = mkElement qn9
  mkattr_qn9 = mkAttr    qn9
  qn10        = mkNsName "element" ns1
  mkelem_qn10 = mkElement qn10
  mkattr_qn10 = mkAttr    qn10
  qn11        = mkNsName "empty" ns1
  mkelem_qn11 = mkElement qn11
  mkattr_qn11 = mkAttr    qn11
  qn12        = mkNsName "except" ns1
  mkelem_qn12 = mkElement qn12
  mkattr_qn12 = mkAttr    qn12
  qn13        = mkNsName "grammar" ns1
  mkelem_qn13 = mkElement qn13
  mkattr_qn13 = mkAttr    qn13
  qn14        = mkNsName "group" ns1
  mkelem_qn14 = mkElement qn14
  mkattr_qn14 = mkAttr    qn14
  qn15        = mkNsName "interleave" ns1
  mkelem_qn15 = mkElement qn15
  mkattr_qn15 = mkAttr    qn15
  qn16        = mkName   "name"
  mkelem_qn16 = mkElement qn16
  mkattr_qn16 = mkAttr    qn16
  qn17        = mkNsName "name" ns1
  mkelem_qn17 = mkElement qn17
  mkattr_qn17 = mkAttr    qn17
  qn18        = mkName   "ns"
  mkelem_qn18 = mkElement qn18
  mkattr_qn18 = mkAttr    qn18
  qn19        = mkNsName "nsName" ns1
  mkelem_qn19 = mkElement qn19
  mkattr_qn19 = mkAttr    qn19
  qn20        = mkNsName "oneOrMore" ns1
  mkelem_qn20 = mkElement qn20
  mkattr_qn20 = mkAttr    qn20
  qn21        = mkNsName "ref" ns1
  mkelem_qn21 = mkElement qn21
  mkattr_qn21 = mkAttr    qn21
  qn22        = mkNsName "start" ns1
  mkelem_qn22 = mkElement qn22
  mkattr_qn22 = mkAttr    qn22
  qn23        = mkNsName "text" ns1
  mkelem_qn23 = mkElement qn23
  mkattr_qn23 = mkAttr    qn23
  qn24        = mkName   "type"
  mkelem_qn24 = mkElement qn24
  mkattr_qn24 = mkAttr    qn24
  qn25        = mkNsName "value" ns1
  mkelem_qn25 = mkElement qn25
  mkattr_qn25 = mkAttr    qn25
  in
  mkRoot []
    [ mkelem_qn13 []
      [ mkelem_qn22 []
        [ mkelem_qn6 []
          [ mkelem_qn6 []
            [ mkelem_qn6 []
              [ mkelem_qn6 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn6 []
                          [ mkelem_qn6 []
                            [ mkelem_qn6 []
                              [ mkelem_qn6 []
                                [ mkelem_qn6 []
                                  [ mkelem_qn6 []
                                    [ mkelem_qn6 []
                                      [ mkelem_qn6 []
                                        [ mkelem_qn6 []
                                          [ mkelem_qn6 []
                                            [ mkelem_qn21
                                              [ mkattr_qn16 [ mkText "24" ]
                                              ] []
                                            , mkelem_qn21
                                              [ mkattr_qn16 [ mkText "25" ]
                                              ] []
                                            ]
                                          , mkelem_qn21
                                            [ mkattr_qn16 [ mkText "26" ]
                                            ] []
                                          ]
                                        , mkelem_qn21
                                          [ mkattr_qn16 [ mkText "27" ]
                                          ] []
                                        ]
                                      , mkelem_qn21
                                        [ mkattr_qn16 [ mkText "28" ]
                                        ] []
                                      ]
                                    , mkelem_qn21
                                      [ mkattr_qn16 [ mkText "29" ]
                                      ] []
                                    ]
                                  , mkelem_qn21
                                    [ mkattr_qn16 [ mkText "30" ]
                                    ] []
                                  ]
                                , mkelem_qn21
                                  [ mkattr_qn16 [ mkText "31" ]
                                  ] []
                                ]
                              , mkelem_qn21
                                [ mkattr_qn16 [ mkText "32" ]
                                ] []
                              ]
                            , mkelem_qn21
                              [ mkattr_qn16 [ mkText "33" ]
                              ] []
                            ]
                          , mkelem_qn21
                            [ mkattr_qn16 [ mkText "34" ]
                            ] []
                          ]
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "35" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "36" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "37" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "38" ]
                    ] []
                  ]
                , mkelem_qn21
                  [ mkattr_qn16 [ mkText "39" ]
                  ] []
                ]
              , mkelem_qn21
                [ mkattr_qn16 [ mkText "42" ]
                ] []
              ]
            , mkelem_qn21
              [ mkattr_qn16 [ mkText "43" ]
              ] []
            ]
          , mkelem_qn21
            [ mkattr_qn16 [ mkText "44" ]
            ] []
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "44" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "grammar"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn21
                          [ mkattr_qn16 [ mkText "11" ]
                          ] []
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "10" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "22" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "23" ]
                      ] []
                    ]
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "43" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "externalRef"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn5 []
                [ mkelem_qn17
                  [ mkattr_qn18 [ mkText "" ]
                  ]
                  [ mkText "href"
                  ]
                , mkelem_qn7
                  [ mkattr_qn24 [ mkText "anyURI" ]
                  , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                  ] []
                ]
              , mkelem_qn14 []
                [ mkelem_qn14 []
                  [ mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "ns"
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  , mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "datatypeLibrary"
                        ]
                      , mkelem_qn7
                        [ mkattr_qn24 [ mkText "anyURI" ]
                        , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                        ] []
                      ]
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn20 []
                    [ mkelem_qn5 []
                      [ mkelem_qn4 []
                        [ mkelem_qn12 []
                          [ mkelem_qn6 []
                            [ mkelem_qn19
                              [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                              ] []
                            , mkelem_qn19
                              [ mkattr_qn18 [ mkText "" ]
                              ] []
                            ]
                          ]
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  ]
                ]
              ]
            , mkelem_qn6 []
              [ mkelem_qn11 [] []
              , mkelem_qn20 []
                [ mkelem_qn21
                  [ mkattr_qn16 [ mkText "15" ]
                  ] []
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "42" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "notAllowed"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn6 []
              [ mkelem_qn11 [] []
              , mkelem_qn20 []
                [ mkelem_qn21
                  [ mkattr_qn16 [ mkText "15" ]
                  ] []
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "39" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "data"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn5 []
                [ mkelem_qn17
                  [ mkattr_qn18 [ mkText "" ]
                  ]
                  [ mkText "type"
                  ]
                , mkelem_qn7
                  [ mkattr_qn24 [ mkText "NCName" ]
                  , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                  ] []
                ]
              , mkelem_qn14 []
                [ mkelem_qn14 []
                  [ mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "ns"
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  , mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "datatypeLibrary"
                        ]
                      , mkelem_qn7
                        [ mkattr_qn24 [ mkText "anyURI" ]
                        , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                        ] []
                      ]
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn20 []
                    [ mkelem_qn5 []
                      [ mkelem_qn4 []
                        [ mkelem_qn12 []
                          [ mkelem_qn6 []
                            [ mkelem_qn19
                              [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                              ] []
                            , mkelem_qn19
                              [ mkattr_qn18 [ mkText "" ]
                              ] []
                            ]
                          ]
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn20 []
                    [ mkelem_qn21
                      [ mkattr_qn16 [ mkText "40" ]
                      ] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "41" ]
                    ] []
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "41" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "except"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn20 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn6 []
                          [ mkelem_qn6 []
                            [ mkelem_qn6 []
                              [ mkelem_qn6 []
                                [ mkelem_qn6 []
                                  [ mkelem_qn6 []
                                    [ mkelem_qn6 []
                                      [ mkelem_qn6 []
                                        [ mkelem_qn6 []
                                          [ mkelem_qn6 []
                                            [ mkelem_qn6 []
                                              [ mkelem_qn6 []
                                                [ mkelem_qn6 []
                                                  [ mkelem_qn6 []
                                                    [ mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "24" ]
                                                      ] []
                                                    , mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "25" ]
                                                      ] []
                                                    ]
                                                  , mkelem_qn21
                                                    [ mkattr_qn16 [ mkText "26" ]
                                                    ] []
                                                  ]
                                                , mkelem_qn21
                                                  [ mkattr_qn16 [ mkText "27" ]
                                                  ] []
                                                ]
                                              , mkelem_qn21
                                                [ mkattr_qn16 [ mkText "28" ]
                                                ] []
                                              ]
                                            , mkelem_qn21
                                              [ mkattr_qn16 [ mkText "29" ]
                                              ] []
                                            ]
                                          , mkelem_qn21
                                            [ mkattr_qn16 [ mkText "30" ]
                                            ] []
                                          ]
                                        , mkelem_qn21
                                          [ mkattr_qn16 [ mkText "31" ]
                                          ] []
                                        ]
                                      , mkelem_qn21
                                        [ mkattr_qn16 [ mkText "32" ]
                                        ] []
                                      ]
                                    , mkelem_qn21
                                      [ mkattr_qn16 [ mkText "33" ]
                                      ] []
                                    ]
                                  , mkelem_qn21
                                    [ mkattr_qn16 [ mkText "34" ]
                                    ] []
                                  ]
                                , mkelem_qn21
                                  [ mkattr_qn16 [ mkText "35" ]
                                  ] []
                                ]
                              , mkelem_qn21
                                [ mkattr_qn16 [ mkText "36" ]
                                ] []
                              ]
                            , mkelem_qn21
                              [ mkattr_qn16 [ mkText "37" ]
                              ] []
                            ]
                          , mkelem_qn21
                            [ mkattr_qn16 [ mkText "38" ]
                            ] []
                          ]
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "39" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "42" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "43" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "44" ]
                    ] []
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "40" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "param"
            ]
          , mkelem_qn14 []
            [ mkelem_qn5 []
              [ mkelem_qn17
                [ mkattr_qn18 [ mkText "" ]
                ]
                [ mkText "name"
                ]
              , mkelem_qn7
                [ mkattr_qn24 [ mkText "NCName" ]
                , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                ] []
              ]
            , mkelem_qn23 [] []
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "38" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "value"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn5 []
                  [ mkelem_qn17
                    [ mkattr_qn18 [ mkText "" ]
                    ]
                    [ mkText "type"
                    ]
                  , mkelem_qn7
                    [ mkattr_qn24 [ mkText "NCName" ]
                    , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                    ] []
                  ]
                ]
              , mkelem_qn14 []
                [ mkelem_qn14 []
                  [ mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "ns"
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  , mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "datatypeLibrary"
                        ]
                      , mkelem_qn7
                        [ mkattr_qn24 [ mkText "anyURI" ]
                        , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                        ] []
                      ]
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn20 []
                    [ mkelem_qn5 []
                      [ mkelem_qn4 []
                        [ mkelem_qn12 []
                          [ mkelem_qn6 []
                            [ mkelem_qn19
                              [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                              ] []
                            , mkelem_qn19
                              [ mkattr_qn18 [ mkText "" ]
                              ] []
                            ]
                          ]
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  ]
                ]
              ]
            , mkelem_qn23 [] []
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "37" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "text"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn6 []
              [ mkelem_qn11 [] []
              , mkelem_qn20 []
                [ mkelem_qn21
                  [ mkattr_qn16 [ mkText "15" ]
                  ] []
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "36" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "empty"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn6 []
              [ mkelem_qn11 [] []
              , mkelem_qn20 []
                [ mkelem_qn21
                  [ mkattr_qn16 [ mkText "15" ]
                  ] []
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "35" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "parentRef"
            ]
          , mkelem_qn14 []
            [ mkelem_qn5 []
              [ mkelem_qn17
                [ mkattr_qn18 [ mkText "" ]
                ]
                [ mkText "name"
                ]
              , mkelem_qn7
                [ mkattr_qn24 [ mkText "NCName" ]
                , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                ] []
              ]
            , mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "34" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "ref"
            ]
          , mkelem_qn14 []
            [ mkelem_qn5 []
              [ mkelem_qn17
                [ mkattr_qn18 [ mkText "" ]
                ]
                [ mkText "name"
                ]
              , mkelem_qn7
                [ mkattr_qn24 [ mkText "NCName" ]
                , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                ] []
              ]
            , mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "33" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "mixed"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn20 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn6 []
                          [ mkelem_qn6 []
                            [ mkelem_qn6 []
                              [ mkelem_qn6 []
                                [ mkelem_qn6 []
                                  [ mkelem_qn6 []
                                    [ mkelem_qn6 []
                                      [ mkelem_qn6 []
                                        [ mkelem_qn6 []
                                          [ mkelem_qn6 []
                                            [ mkelem_qn6 []
                                              [ mkelem_qn6 []
                                                [ mkelem_qn6 []
                                                  [ mkelem_qn6 []
                                                    [ mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "24" ]
                                                      ] []
                                                    , mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "25" ]
                                                      ] []
                                                    ]
                                                  , mkelem_qn21
                                                    [ mkattr_qn16 [ mkText "26" ]
                                                    ] []
                                                  ]
                                                , mkelem_qn21
                                                  [ mkattr_qn16 [ mkText "27" ]
                                                  ] []
                                                ]
                                              , mkelem_qn21
                                                [ mkattr_qn16 [ mkText "28" ]
                                                ] []
                                              ]
                                            , mkelem_qn21
                                              [ mkattr_qn16 [ mkText "29" ]
                                              ] []
                                            ]
                                          , mkelem_qn21
                                            [ mkattr_qn16 [ mkText "30" ]
                                            ] []
                                          ]
                                        , mkelem_qn21
                                          [ mkattr_qn16 [ mkText "31" ]
                                          ] []
                                        ]
                                      , mkelem_qn21
                                        [ mkattr_qn16 [ mkText "32" ]
                                        ] []
                                      ]
                                    , mkelem_qn21
                                      [ mkattr_qn16 [ mkText "33" ]
                                      ] []
                                    ]
                                  , mkelem_qn21
                                    [ mkattr_qn16 [ mkText "34" ]
                                    ] []
                                  ]
                                , mkelem_qn21
                                  [ mkattr_qn16 [ mkText "35" ]
                                  ] []
                                ]
                              , mkelem_qn21
                                [ mkattr_qn16 [ mkText "36" ]
                                ] []
                              ]
                            , mkelem_qn21
                              [ mkattr_qn16 [ mkText "37" ]
                              ] []
                            ]
                          , mkelem_qn21
                            [ mkattr_qn16 [ mkText "38" ]
                            ] []
                          ]
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "39" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "42" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "43" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "44" ]
                    ] []
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "32" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "list"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn20 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn6 []
                          [ mkelem_qn6 []
                            [ mkelem_qn6 []
                              [ mkelem_qn6 []
                                [ mkelem_qn6 []
                                  [ mkelem_qn6 []
                                    [ mkelem_qn6 []
                                      [ mkelem_qn6 []
                                        [ mkelem_qn6 []
                                          [ mkelem_qn6 []
                                            [ mkelem_qn6 []
                                              [ mkelem_qn6 []
                                                [ mkelem_qn6 []
                                                  [ mkelem_qn6 []
                                                    [ mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "24" ]
                                                      ] []
                                                    , mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "25" ]
                                                      ] []
                                                    ]
                                                  , mkelem_qn21
                                                    [ mkattr_qn16 [ mkText "26" ]
                                                    ] []
                                                  ]
                                                , mkelem_qn21
                                                  [ mkattr_qn16 [ mkText "27" ]
                                                  ] []
                                                ]
                                              , mkelem_qn21
                                                [ mkattr_qn16 [ mkText "28" ]
                                                ] []
                                              ]
                                            , mkelem_qn21
                                              [ mkattr_qn16 [ mkText "29" ]
                                              ] []
                                            ]
                                          , mkelem_qn21
                                            [ mkattr_qn16 [ mkText "30" ]
                                            ] []
                                          ]
                                        , mkelem_qn21
                                          [ mkattr_qn16 [ mkText "31" ]
                                          ] []
                                        ]
                                      , mkelem_qn21
                                        [ mkattr_qn16 [ mkText "32" ]
                                        ] []
                                      ]
                                    , mkelem_qn21
                                      [ mkattr_qn16 [ mkText "33" ]
                                      ] []
                                    ]
                                  , mkelem_qn21
                                    [ mkattr_qn16 [ mkText "34" ]
                                    ] []
                                  ]
                                , mkelem_qn21
                                  [ mkattr_qn16 [ mkText "35" ]
                                  ] []
                                ]
                              , mkelem_qn21
                                [ mkattr_qn16 [ mkText "36" ]
                                ] []
                              ]
                            , mkelem_qn21
                              [ mkattr_qn16 [ mkText "37" ]
                              ] []
                            ]
                          , mkelem_qn21
                            [ mkattr_qn16 [ mkText "38" ]
                            ] []
                          ]
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "39" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "42" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "43" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "44" ]
                    ] []
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "31" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "oneOrMore"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn20 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn6 []
                          [ mkelem_qn6 []
                            [ mkelem_qn6 []
                              [ mkelem_qn6 []
                                [ mkelem_qn6 []
                                  [ mkelem_qn6 []
                                    [ mkelem_qn6 []
                                      [ mkelem_qn6 []
                                        [ mkelem_qn6 []
                                          [ mkelem_qn6 []
                                            [ mkelem_qn6 []
                                              [ mkelem_qn6 []
                                                [ mkelem_qn6 []
                                                  [ mkelem_qn6 []
                                                    [ mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "24" ]
                                                      ] []
                                                    , mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "25" ]
                                                      ] []
                                                    ]
                                                  , mkelem_qn21
                                                    [ mkattr_qn16 [ mkText "26" ]
                                                    ] []
                                                  ]
                                                , mkelem_qn21
                                                  [ mkattr_qn16 [ mkText "27" ]
                                                  ] []
                                                ]
                                              , mkelem_qn21
                                                [ mkattr_qn16 [ mkText "28" ]
                                                ] []
                                              ]
                                            , mkelem_qn21
                                              [ mkattr_qn16 [ mkText "29" ]
                                              ] []
                                            ]
                                          , mkelem_qn21
                                            [ mkattr_qn16 [ mkText "30" ]
                                            ] []
                                          ]
                                        , mkelem_qn21
                                          [ mkattr_qn16 [ mkText "31" ]
                                          ] []
                                        ]
                                      , mkelem_qn21
                                        [ mkattr_qn16 [ mkText "32" ]
                                        ] []
                                      ]
                                    , mkelem_qn21
                                      [ mkattr_qn16 [ mkText "33" ]
                                      ] []
                                    ]
                                  , mkelem_qn21
                                    [ mkattr_qn16 [ mkText "34" ]
                                    ] []
                                  ]
                                , mkelem_qn21
                                  [ mkattr_qn16 [ mkText "35" ]
                                  ] []
                                ]
                              , mkelem_qn21
                                [ mkattr_qn16 [ mkText "36" ]
                                ] []
                              ]
                            , mkelem_qn21
                              [ mkattr_qn16 [ mkText "37" ]
                              ] []
                            ]
                          , mkelem_qn21
                            [ mkattr_qn16 [ mkText "38" ]
                            ] []
                          ]
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "39" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "42" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "43" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "44" ]
                    ] []
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "30" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "zeroOrMore"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn20 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn6 []
                          [ mkelem_qn6 []
                            [ mkelem_qn6 []
                              [ mkelem_qn6 []
                                [ mkelem_qn6 []
                                  [ mkelem_qn6 []
                                    [ mkelem_qn6 []
                                      [ mkelem_qn6 []
                                        [ mkelem_qn6 []
                                          [ mkelem_qn6 []
                                            [ mkelem_qn6 []
                                              [ mkelem_qn6 []
                                                [ mkelem_qn6 []
                                                  [ mkelem_qn6 []
                                                    [ mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "24" ]
                                                      ] []
                                                    , mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "25" ]
                                                      ] []
                                                    ]
                                                  , mkelem_qn21
                                                    [ mkattr_qn16 [ mkText "26" ]
                                                    ] []
                                                  ]
                                                , mkelem_qn21
                                                  [ mkattr_qn16 [ mkText "27" ]
                                                  ] []
                                                ]
                                              , mkelem_qn21
                                                [ mkattr_qn16 [ mkText "28" ]
                                                ] []
                                              ]
                                            , mkelem_qn21
                                              [ mkattr_qn16 [ mkText "29" ]
                                              ] []
                                            ]
                                          , mkelem_qn21
                                            [ mkattr_qn16 [ mkText "30" ]
                                            ] []
                                          ]
                                        , mkelem_qn21
                                          [ mkattr_qn16 [ mkText "31" ]
                                          ] []
                                        ]
                                      , mkelem_qn21
                                        [ mkattr_qn16 [ mkText "32" ]
                                        ] []
                                      ]
                                    , mkelem_qn21
                                      [ mkattr_qn16 [ mkText "33" ]
                                      ] []
                                    ]
                                  , mkelem_qn21
                                    [ mkattr_qn16 [ mkText "34" ]
                                    ] []
                                  ]
                                , mkelem_qn21
                                  [ mkattr_qn16 [ mkText "35" ]
                                  ] []
                                ]
                              , mkelem_qn21
                                [ mkattr_qn16 [ mkText "36" ]
                                ] []
                              ]
                            , mkelem_qn21
                              [ mkattr_qn16 [ mkText "37" ]
                              ] []
                            ]
                          , mkelem_qn21
                            [ mkattr_qn16 [ mkText "38" ]
                            ] []
                          ]
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "39" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "42" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "43" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "44" ]
                    ] []
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "29" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "optional"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn20 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn6 []
                          [ mkelem_qn6 []
                            [ mkelem_qn6 []
                              [ mkelem_qn6 []
                                [ mkelem_qn6 []
                                  [ mkelem_qn6 []
                                    [ mkelem_qn6 []
                                      [ mkelem_qn6 []
                                        [ mkelem_qn6 []
                                          [ mkelem_qn6 []
                                            [ mkelem_qn6 []
                                              [ mkelem_qn6 []
                                                [ mkelem_qn6 []
                                                  [ mkelem_qn6 []
                                                    [ mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "24" ]
                                                      ] []
                                                    , mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "25" ]
                                                      ] []
                                                    ]
                                                  , mkelem_qn21
                                                    [ mkattr_qn16 [ mkText "26" ]
                                                    ] []
                                                  ]
                                                , mkelem_qn21
                                                  [ mkattr_qn16 [ mkText "27" ]
                                                  ] []
                                                ]
                                              , mkelem_qn21
                                                [ mkattr_qn16 [ mkText "28" ]
                                                ] []
                                              ]
                                            , mkelem_qn21
                                              [ mkattr_qn16 [ mkText "29" ]
                                              ] []
                                            ]
                                          , mkelem_qn21
                                            [ mkattr_qn16 [ mkText "30" ]
                                            ] []
                                          ]
                                        , mkelem_qn21
                                          [ mkattr_qn16 [ mkText "31" ]
                                          ] []
                                        ]
                                      , mkelem_qn21
                                        [ mkattr_qn16 [ mkText "32" ]
                                        ] []
                                      ]
                                    , mkelem_qn21
                                      [ mkattr_qn16 [ mkText "33" ]
                                      ] []
                                    ]
                                  , mkelem_qn21
                                    [ mkattr_qn16 [ mkText "34" ]
                                    ] []
                                  ]
                                , mkelem_qn21
                                  [ mkattr_qn16 [ mkText "35" ]
                                  ] []
                                ]
                              , mkelem_qn21
                                [ mkattr_qn16 [ mkText "36" ]
                                ] []
                              ]
                            , mkelem_qn21
                              [ mkattr_qn16 [ mkText "37" ]
                              ] []
                            ]
                          , mkelem_qn21
                            [ mkattr_qn16 [ mkText "38" ]
                            ] []
                          ]
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "39" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "42" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "43" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "44" ]
                    ] []
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "28" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "choice"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn20 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn6 []
                          [ mkelem_qn6 []
                            [ mkelem_qn6 []
                              [ mkelem_qn6 []
                                [ mkelem_qn6 []
                                  [ mkelem_qn6 []
                                    [ mkelem_qn6 []
                                      [ mkelem_qn6 []
                                        [ mkelem_qn6 []
                                          [ mkelem_qn6 []
                                            [ mkelem_qn6 []
                                              [ mkelem_qn6 []
                                                [ mkelem_qn6 []
                                                  [ mkelem_qn6 []
                                                    [ mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "24" ]
                                                      ] []
                                                    , mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "25" ]
                                                      ] []
                                                    ]
                                                  , mkelem_qn21
                                                    [ mkattr_qn16 [ mkText "26" ]
                                                    ] []
                                                  ]
                                                , mkelem_qn21
                                                  [ mkattr_qn16 [ mkText "27" ]
                                                  ] []
                                                ]
                                              , mkelem_qn21
                                                [ mkattr_qn16 [ mkText "28" ]
                                                ] []
                                              ]
                                            , mkelem_qn21
                                              [ mkattr_qn16 [ mkText "29" ]
                                              ] []
                                            ]
                                          , mkelem_qn21
                                            [ mkattr_qn16 [ mkText "30" ]
                                            ] []
                                          ]
                                        , mkelem_qn21
                                          [ mkattr_qn16 [ mkText "31" ]
                                          ] []
                                        ]
                                      , mkelem_qn21
                                        [ mkattr_qn16 [ mkText "32" ]
                                        ] []
                                      ]
                                    , mkelem_qn21
                                      [ mkattr_qn16 [ mkText "33" ]
                                      ] []
                                    ]
                                  , mkelem_qn21
                                    [ mkattr_qn16 [ mkText "34" ]
                                    ] []
                                  ]
                                , mkelem_qn21
                                  [ mkattr_qn16 [ mkText "35" ]
                                  ] []
                                ]
                              , mkelem_qn21
                                [ mkattr_qn16 [ mkText "36" ]
                                ] []
                              ]
                            , mkelem_qn21
                              [ mkattr_qn16 [ mkText "37" ]
                              ] []
                            ]
                          , mkelem_qn21
                            [ mkattr_qn16 [ mkText "38" ]
                            ] []
                          ]
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "39" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "42" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "43" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "44" ]
                    ] []
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "27" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "interleave"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn20 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn6 []
                          [ mkelem_qn6 []
                            [ mkelem_qn6 []
                              [ mkelem_qn6 []
                                [ mkelem_qn6 []
                                  [ mkelem_qn6 []
                                    [ mkelem_qn6 []
                                      [ mkelem_qn6 []
                                        [ mkelem_qn6 []
                                          [ mkelem_qn6 []
                                            [ mkelem_qn6 []
                                              [ mkelem_qn6 []
                                                [ mkelem_qn6 []
                                                  [ mkelem_qn6 []
                                                    [ mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "24" ]
                                                      ] []
                                                    , mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "25" ]
                                                      ] []
                                                    ]
                                                  , mkelem_qn21
                                                    [ mkattr_qn16 [ mkText "26" ]
                                                    ] []
                                                  ]
                                                , mkelem_qn21
                                                  [ mkattr_qn16 [ mkText "27" ]
                                                  ] []
                                                ]
                                              , mkelem_qn21
                                                [ mkattr_qn16 [ mkText "28" ]
                                                ] []
                                              ]
                                            , mkelem_qn21
                                              [ mkattr_qn16 [ mkText "29" ]
                                              ] []
                                            ]
                                          , mkelem_qn21
                                            [ mkattr_qn16 [ mkText "30" ]
                                            ] []
                                          ]
                                        , mkelem_qn21
                                          [ mkattr_qn16 [ mkText "31" ]
                                          ] []
                                        ]
                                      , mkelem_qn21
                                        [ mkattr_qn16 [ mkText "32" ]
                                        ] []
                                      ]
                                    , mkelem_qn21
                                      [ mkattr_qn16 [ mkText "33" ]
                                      ] []
                                    ]
                                  , mkelem_qn21
                                    [ mkattr_qn16 [ mkText "34" ]
                                    ] []
                                  ]
                                , mkelem_qn21
                                  [ mkattr_qn16 [ mkText "35" ]
                                  ] []
                                ]
                              , mkelem_qn21
                                [ mkattr_qn16 [ mkText "36" ]
                                ] []
                              ]
                            , mkelem_qn21
                              [ mkattr_qn16 [ mkText "37" ]
                              ] []
                            ]
                          , mkelem_qn21
                            [ mkattr_qn16 [ mkText "38" ]
                            ] []
                          ]
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "39" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "42" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "43" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "44" ]
                    ] []
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "26" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "group"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn20 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn6 []
                          [ mkelem_qn6 []
                            [ mkelem_qn6 []
                              [ mkelem_qn6 []
                                [ mkelem_qn6 []
                                  [ mkelem_qn6 []
                                    [ mkelem_qn6 []
                                      [ mkelem_qn6 []
                                        [ mkelem_qn6 []
                                          [ mkelem_qn6 []
                                            [ mkelem_qn6 []
                                              [ mkelem_qn6 []
                                                [ mkelem_qn6 []
                                                  [ mkelem_qn6 []
                                                    [ mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "24" ]
                                                      ] []
                                                    , mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "25" ]
                                                      ] []
                                                    ]
                                                  , mkelem_qn21
                                                    [ mkattr_qn16 [ mkText "26" ]
                                                    ] []
                                                  ]
                                                , mkelem_qn21
                                                  [ mkattr_qn16 [ mkText "27" ]
                                                  ] []
                                                ]
                                              , mkelem_qn21
                                                [ mkattr_qn16 [ mkText "28" ]
                                                ] []
                                              ]
                                            , mkelem_qn21
                                              [ mkattr_qn16 [ mkText "29" ]
                                              ] []
                                            ]
                                          , mkelem_qn21
                                            [ mkattr_qn16 [ mkText "30" ]
                                            ] []
                                          ]
                                        , mkelem_qn21
                                          [ mkattr_qn16 [ mkText "31" ]
                                          ] []
                                        ]
                                      , mkelem_qn21
                                        [ mkattr_qn16 [ mkText "32" ]
                                        ] []
                                      ]
                                    , mkelem_qn21
                                      [ mkattr_qn16 [ mkText "33" ]
                                      ] []
                                    ]
                                  , mkelem_qn21
                                    [ mkattr_qn16 [ mkText "34" ]
                                    ] []
                                  ]
                                , mkelem_qn21
                                  [ mkattr_qn16 [ mkText "35" ]
                                  ] []
                                ]
                              , mkelem_qn21
                                [ mkattr_qn16 [ mkText "36" ]
                                ] []
                              ]
                            , mkelem_qn21
                              [ mkattr_qn16 [ mkText "37" ]
                              ] []
                            ]
                          , mkelem_qn21
                            [ mkattr_qn16 [ mkText "38" ]
                            ] []
                          ]
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "39" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "42" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "43" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "44" ]
                    ] []
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "25" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "attribute"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn14 []
                  [ mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "ns"
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  , mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "datatypeLibrary"
                        ]
                      , mkelem_qn7
                        [ mkattr_qn24 [ mkText "anyURI" ]
                        , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                        ] []
                      ]
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn20 []
                    [ mkelem_qn5 []
                      [ mkelem_qn4 []
                        [ mkelem_qn12 []
                          [ mkelem_qn6 []
                            [ mkelem_qn19
                              [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                              ] []
                            , mkelem_qn19
                              [ mkattr_qn18 [ mkText "" ]
                              ] []
                            ]
                          ]
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn5 []
                  [ mkelem_qn17
                    [ mkattr_qn18 [ mkText "" ]
                    ]
                    [ mkText "name"
                    ]
                  , mkelem_qn7
                    [ mkattr_qn24 [ mkText "QName" ]
                    , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                    ] []
                  ]
                , mkelem_qn15 []
                  [ mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn20 []
                      [ mkelem_qn21
                        [ mkattr_qn16 [ mkText "15" ]
                        ] []
                      ]
                    ]
                  , mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn21
                          [ mkattr_qn16 [ mkText "17" ]
                          ] []
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "18" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "19" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "20" ]
                      ] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn6 []
                          [ mkelem_qn6 []
                            [ mkelem_qn6 []
                              [ mkelem_qn6 []
                                [ mkelem_qn6 []
                                  [ mkelem_qn6 []
                                    [ mkelem_qn6 []
                                      [ mkelem_qn6 []
                                        [ mkelem_qn6 []
                                          [ mkelem_qn6 []
                                            [ mkelem_qn6 []
                                              [ mkelem_qn6 []
                                                [ mkelem_qn6 []
                                                  [ mkelem_qn6 []
                                                    [ mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "24" ]
                                                      ] []
                                                    , mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "25" ]
                                                      ] []
                                                    ]
                                                  , mkelem_qn21
                                                    [ mkattr_qn16 [ mkText "26" ]
                                                    ] []
                                                  ]
                                                , mkelem_qn21
                                                  [ mkattr_qn16 [ mkText "27" ]
                                                  ] []
                                                ]
                                              , mkelem_qn21
                                                [ mkattr_qn16 [ mkText "28" ]
                                                ] []
                                              ]
                                            , mkelem_qn21
                                              [ mkattr_qn16 [ mkText "29" ]
                                              ] []
                                            ]
                                          , mkelem_qn21
                                            [ mkattr_qn16 [ mkText "30" ]
                                            ] []
                                          ]
                                        , mkelem_qn21
                                          [ mkattr_qn16 [ mkText "31" ]
                                          ] []
                                        ]
                                      , mkelem_qn21
                                        [ mkattr_qn16 [ mkText "32" ]
                                        ] []
                                      ]
                                    , mkelem_qn21
                                      [ mkattr_qn16 [ mkText "33" ]
                                      ] []
                                    ]
                                  , mkelem_qn21
                                    [ mkattr_qn16 [ mkText "34" ]
                                    ] []
                                  ]
                                , mkelem_qn21
                                  [ mkattr_qn16 [ mkText "35" ]
                                  ] []
                                ]
                              , mkelem_qn21
                                [ mkattr_qn16 [ mkText "36" ]
                                ] []
                              ]
                            , mkelem_qn21
                              [ mkattr_qn16 [ mkText "37" ]
                              ] []
                            ]
                          , mkelem_qn21
                            [ mkattr_qn16 [ mkText "38" ]
                            ] []
                          ]
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "39" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "42" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "43" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "44" ]
                    ] []
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "24" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "element"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn6 []
                [ mkelem_qn5 []
                  [ mkelem_qn17
                    [ mkattr_qn18 [ mkText "" ]
                    ]
                    [ mkText "name"
                    ]
                  , mkelem_qn7
                    [ mkattr_qn24 [ mkText "QName" ]
                    , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                    ] []
                  ]
                , mkelem_qn15 []
                  [ mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn20 []
                      [ mkelem_qn21
                        [ mkattr_qn16 [ mkText "15" ]
                        ] []
                      ]
                    ]
                  , mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn21
                          [ mkattr_qn16 [ mkText "17" ]
                          ] []
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "18" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "19" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "20" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn14 []
                [ mkelem_qn14 []
                  [ mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "ns"
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  , mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "datatypeLibrary"
                        ]
                      , mkelem_qn7
                        [ mkattr_qn24 [ mkText "anyURI" ]
                        , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                        ] []
                      ]
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn20 []
                    [ mkelem_qn5 []
                      [ mkelem_qn4 []
                        [ mkelem_qn12 []
                          [ mkelem_qn6 []
                            [ mkelem_qn19
                              [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                              ] []
                            , mkelem_qn19
                              [ mkattr_qn18 [ mkText "" ]
                              ] []
                            ]
                          ]
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn20 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn6 []
                          [ mkelem_qn6 []
                            [ mkelem_qn6 []
                              [ mkelem_qn6 []
                                [ mkelem_qn6 []
                                  [ mkelem_qn6 []
                                    [ mkelem_qn6 []
                                      [ mkelem_qn6 []
                                        [ mkelem_qn6 []
                                          [ mkelem_qn6 []
                                            [ mkelem_qn6 []
                                              [ mkelem_qn6 []
                                                [ mkelem_qn6 []
                                                  [ mkelem_qn6 []
                                                    [ mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "24" ]
                                                      ] []
                                                    , mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "25" ]
                                                      ] []
                                                    ]
                                                  , mkelem_qn21
                                                    [ mkattr_qn16 [ mkText "26" ]
                                                    ] []
                                                  ]
                                                , mkelem_qn21
                                                  [ mkattr_qn16 [ mkText "27" ]
                                                  ] []
                                                ]
                                              , mkelem_qn21
                                                [ mkattr_qn16 [ mkText "28" ]
                                                ] []
                                              ]
                                            , mkelem_qn21
                                              [ mkattr_qn16 [ mkText "29" ]
                                              ] []
                                            ]
                                          , mkelem_qn21
                                            [ mkattr_qn16 [ mkText "30" ]
                                            ] []
                                          ]
                                        , mkelem_qn21
                                          [ mkattr_qn16 [ mkText "31" ]
                                          ] []
                                        ]
                                      , mkelem_qn21
                                        [ mkattr_qn16 [ mkText "32" ]
                                        ] []
                                      ]
                                    , mkelem_qn21
                                      [ mkattr_qn16 [ mkText "33" ]
                                      ] []
                                    ]
                                  , mkelem_qn21
                                    [ mkattr_qn16 [ mkText "34" ]
                                    ] []
                                  ]
                                , mkelem_qn21
                                  [ mkattr_qn16 [ mkText "35" ]
                                  ] []
                                ]
                              , mkelem_qn21
                                [ mkattr_qn16 [ mkText "36" ]
                                ] []
                              ]
                            , mkelem_qn21
                              [ mkattr_qn16 [ mkText "37" ]
                              ] []
                            ]
                          , mkelem_qn21
                            [ mkattr_qn16 [ mkText "38" ]
                            ] []
                          ]
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "39" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "42" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "43" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "44" ]
                    ] []
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "23" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "include"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn5 []
                [ mkelem_qn17
                  [ mkattr_qn18 [ mkText "" ]
                  ]
                  [ mkText "href"
                  ]
                , mkelem_qn7
                  [ mkattr_qn24 [ mkText "anyURI" ]
                  , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                  ] []
                ]
              , mkelem_qn14 []
                [ mkelem_qn14 []
                  [ mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "ns"
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  , mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "datatypeLibrary"
                        ]
                      , mkelem_qn7
                        [ mkattr_qn24 [ mkText "anyURI" ]
                        , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                        ] []
                      ]
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn20 []
                    [ mkelem_qn5 []
                      [ mkelem_qn4 []
                        [ mkelem_qn12 []
                          [ mkelem_qn6 []
                            [ mkelem_qn19
                              [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                              ] []
                            , mkelem_qn19
                              [ mkattr_qn18 [ mkText "" ]
                              ] []
                            ]
                          ]
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn21
                        [ mkattr_qn16 [ mkText "11" ]
                        ] []
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "10" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "21" ]
                      ] []
                    ]
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "22" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "div"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn21
                          [ mkattr_qn16 [ mkText "11" ]
                          ] []
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "10" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "22" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "23" ]
                      ] []
                    ]
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "21" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "div"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn21
                        [ mkattr_qn16 [ mkText "11" ]
                        ] []
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "10" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "21" ]
                      ] []
                    ]
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "20" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "choice"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn20 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn21
                        [ mkattr_qn16 [ mkText "17" ]
                        ] []
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "18" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "19" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "20" ]
                    ] []
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "19" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "nsName"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn21
                  [ mkattr_qn16 [ mkText "16" ]
                  ] []
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "18" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "anyName"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn21
                  [ mkattr_qn16 [ mkText "16" ]
                  ] []
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "17" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "name"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "ns"
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "datatypeLibrary"
                      ]
                    , mkelem_qn7
                      [ mkattr_qn24 [ mkText "anyURI" ]
                      , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                      ] []
                    ]
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 []
                      [ mkelem_qn12 []
                        [ mkelem_qn6 []
                          [ mkelem_qn19
                            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                            ] []
                          , mkelem_qn19
                            [ mkattr_qn18 [ mkText "" ]
                            ] []
                          ]
                        ]
                      ]
                    , mkelem_qn23 [] []
                    ]
                  ]
                ]
              ]
            , mkelem_qn7
              [ mkattr_qn24 [ mkText "QName" ]
              , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
              ] []
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "16" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "except"
            ]
          , mkelem_qn15 []
            [ mkelem_qn6 []
              [ mkelem_qn11 [] []
              , mkelem_qn20 []
                [ mkelem_qn21
                  [ mkattr_qn16 [ mkText "15" ]
                  ] []
                ]
              ]
            , mkelem_qn20 []
              [ mkelem_qn6 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn21
                      [ mkattr_qn16 [ mkText "17" ]
                      ] []
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "18" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "19" ]
                    ] []
                  ]
                , mkelem_qn21
                  [ mkattr_qn16 [ mkText "20" ]
                  ] []
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "15" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn4 []
            [ mkelem_qn12 []
              [ mkelem_qn19
                [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                ] []
              ]
            ]
          , mkelem_qn6 []
            [ mkelem_qn11 [] []
            , mkelem_qn20 []
              [ mkelem_qn6 []
                [ mkelem_qn6 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 [] []
                    , mkelem_qn23 [] []
                    ]
                  , mkelem_qn23 [] []
                  ]
                , mkelem_qn21
                  [ mkattr_qn16 [ mkText "0" ]
                  ] []
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "0" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn4 [] []
          , mkelem_qn6 []
            [ mkelem_qn11 [] []
            , mkelem_qn20 []
              [ mkelem_qn6 []
                [ mkelem_qn6 []
                  [ mkelem_qn5 []
                    [ mkelem_qn4 [] []
                    , mkelem_qn23 [] []
                    ]
                  , mkelem_qn23 [] []
                  ]
                , mkelem_qn21
                  [ mkattr_qn16 [ mkText "0" ]
                  ] []
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "10" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "define"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn14 []
                [ mkelem_qn5 []
                  [ mkelem_qn17
                    [ mkattr_qn18 [ mkText "" ]
                    ]
                    [ mkText "name"
                    ]
                  , mkelem_qn7
                    [ mkattr_qn24 [ mkText "NCName" ]
                    , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                    ] []
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn5 []
                    [ mkelem_qn17
                      [ mkattr_qn18 [ mkText "" ]
                      ]
                      [ mkText "combine"
                      ]
                    , mkelem_qn6 []
                      [ mkelem_qn25
                        [ mkattr_qn1 [ mkText "http://www.w3.org/XML/1998/namespace" ]
                        , mkattr_qn3 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                        , mkattr_qn2 [ mkText "file:///home/theo/haskell/hxt/hxt-relaxng/src/Text/XML/HXT/RelaxNG/schema2hs/Schema.rng" ]
                        , mkattr_qn8 [ mkText "" ]
                        , mkattr_qn24 [ mkText "token" ]
                        , mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                        ]
                        [ mkText "choice"
                        ]
                      , mkelem_qn25
                        [ mkattr_qn1 [ mkText "http://www.w3.org/XML/1998/namespace" ]
                        , mkattr_qn3 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                        , mkattr_qn2 [ mkText "file:///home/theo/haskell/hxt/hxt-relaxng/src/Text/XML/HXT/RelaxNG/schema2hs/Schema.rng" ]
                        , mkattr_qn8 [ mkText "" ]
                        , mkattr_qn24 [ mkText "token" ]
                        , mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                        ]
                        [ mkText "interleave"
                        ]
                      ]
                    ]
                  ]
                ]
              , mkelem_qn14 []
                [ mkelem_qn14 []
                  [ mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "ns"
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  , mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "datatypeLibrary"
                        ]
                      , mkelem_qn7
                        [ mkattr_qn24 [ mkText "anyURI" ]
                        , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                        ] []
                      ]
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn20 []
                    [ mkelem_qn5 []
                      [ mkelem_qn4 []
                        [ mkelem_qn12 []
                          [ mkelem_qn6 []
                            [ mkelem_qn19
                              [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                              ] []
                            , mkelem_qn19
                              [ mkattr_qn18 [ mkText "" ]
                              ] []
                            ]
                          ]
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn20 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn6 []
                          [ mkelem_qn6 []
                            [ mkelem_qn6 []
                              [ mkelem_qn6 []
                                [ mkelem_qn6 []
                                  [ mkelem_qn6 []
                                    [ mkelem_qn6 []
                                      [ mkelem_qn6 []
                                        [ mkelem_qn6 []
                                          [ mkelem_qn6 []
                                            [ mkelem_qn6 []
                                              [ mkelem_qn6 []
                                                [ mkelem_qn6 []
                                                  [ mkelem_qn6 []
                                                    [ mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "24" ]
                                                      ] []
                                                    , mkelem_qn21
                                                      [ mkattr_qn16 [ mkText "25" ]
                                                      ] []
                                                    ]
                                                  , mkelem_qn21
                                                    [ mkattr_qn16 [ mkText "26" ]
                                                    ] []
                                                  ]
                                                , mkelem_qn21
                                                  [ mkattr_qn16 [ mkText "27" ]
                                                  ] []
                                                ]
                                              , mkelem_qn21
                                                [ mkattr_qn16 [ mkText "28" ]
                                                ] []
                                              ]
                                            , mkelem_qn21
                                              [ mkattr_qn16 [ mkText "29" ]
                                              ] []
                                            ]
                                          , mkelem_qn21
                                            [ mkattr_qn16 [ mkText "30" ]
                                            ] []
                                          ]
                                        , mkelem_qn21
                                          [ mkattr_qn16 [ mkText "31" ]
                                          ] []
                                        ]
                                      , mkelem_qn21
                                        [ mkattr_qn16 [ mkText "32" ]
                                        ] []
                                      ]
                                    , mkelem_qn21
                                      [ mkattr_qn16 [ mkText "33" ]
                                      ] []
                                    ]
                                  , mkelem_qn21
                                    [ mkattr_qn16 [ mkText "34" ]
                                    ] []
                                  ]
                                , mkelem_qn21
                                  [ mkattr_qn16 [ mkText "35" ]
                                  ] []
                                ]
                              , mkelem_qn21
                                [ mkattr_qn16 [ mkText "36" ]
                                ] []
                              ]
                            , mkelem_qn21
                              [ mkattr_qn16 [ mkText "37" ]
                              ] []
                            ]
                          , mkelem_qn21
                            [ mkattr_qn16 [ mkText "38" ]
                            ] []
                          ]
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "39" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "42" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "43" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "44" ]
                    ] []
                  ]
                ]
              ]
            ]
          ]
        ]
      , mkelem_qn9
        [ mkattr_qn16 [ mkText "11" ]
        ]
        [ mkelem_qn10 []
          [ mkelem_qn17
            [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
            ]
            [ mkText "start"
            ]
          , mkelem_qn14 []
            [ mkelem_qn14 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn5 []
                  [ mkelem_qn17
                    [ mkattr_qn18 [ mkText "" ]
                    ]
                    [ mkText "combine"
                    ]
                  , mkelem_qn6 []
                    [ mkelem_qn25
                      [ mkattr_qn1 [ mkText "http://www.w3.org/XML/1998/namespace" ]
                      , mkattr_qn3 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                      , mkattr_qn2 [ mkText "file:///home/theo/haskell/hxt/hxt-relaxng/src/Text/XML/HXT/RelaxNG/schema2hs/Schema.rng" ]
                      , mkattr_qn8 [ mkText "" ]
                      , mkattr_qn24 [ mkText "token" ]
                      , mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                      ]
                      [ mkText "choice"
                      ]
                    , mkelem_qn25
                      [ mkattr_qn1 [ mkText "http://www.w3.org/XML/1998/namespace" ]
                      , mkattr_qn3 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                      , mkattr_qn2 [ mkText "file:///home/theo/haskell/hxt/hxt-relaxng/src/Text/XML/HXT/RelaxNG/schema2hs/Schema.rng" ]
                      , mkattr_qn8 [ mkText "" ]
                      , mkattr_qn24 [ mkText "token" ]
                      , mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                      ]
                      [ mkText "interleave"
                      ]
                    ]
                  ]
                ]
              , mkelem_qn14 []
                [ mkelem_qn14 []
                  [ mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "ns"
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  , mkelem_qn6 []
                    [ mkelem_qn11 [] []
                    , mkelem_qn5 []
                      [ mkelem_qn17
                        [ mkattr_qn18 [ mkText "" ]
                        ]
                        [ mkText "datatypeLibrary"
                        ]
                      , mkelem_qn7
                        [ mkattr_qn24 [ mkText "anyURI" ]
                        , mkattr_qn8 [ mkText "http://www.w3.org/2001/XMLSchema-datatypes" ]
                        ] []
                      ]
                    ]
                  ]
                , mkelem_qn6 []
                  [ mkelem_qn11 [] []
                  , mkelem_qn20 []
                    [ mkelem_qn5 []
                      [ mkelem_qn4 []
                        [ mkelem_qn12 []
                          [ mkelem_qn6 []
                            [ mkelem_qn19
                              [ mkattr_qn18 [ mkText "http://relaxng.org/ns/structure/1.0" ]
                              ] []
                            , mkelem_qn19
                              [ mkattr_qn18 [ mkText "" ]
                              ] []
                            ]
                          ]
                        ]
                      , mkelem_qn23 [] []
                      ]
                    ]
                  ]
                ]
              ]
            , mkelem_qn15 []
              [ mkelem_qn6 []
                [ mkelem_qn11 [] []
                , mkelem_qn20 []
                  [ mkelem_qn21
                    [ mkattr_qn16 [ mkText "15" ]
                    ] []
                  ]
                ]
              , mkelem_qn6 []
                [ mkelem_qn6 []
                  [ mkelem_qn6 []
                    [ mkelem_qn6 []
                      [ mkelem_qn6 []
                        [ mkelem_qn6 []
                          [ mkelem_qn6 []
                            [ mkelem_qn6 []
                              [ mkelem_qn6 []
                                [ mkelem_qn6 []
                                  [ mkelem_qn6 []
                                    [ mkelem_qn6 []
                                      [ mkelem_qn6 []
                                        [ mkelem_qn6 []
                                          [ mkelem_qn6 []
                                            [ mkelem_qn6 []
                                              [ mkelem_qn6 []
                                                [ mkelem_qn6 []
                                                  [ mkelem_qn21
                                                    [ mkattr_qn16 [ mkText "24" ]
                                                    ] []
                                                  , mkelem_qn21
                                                    [ mkattr_qn16 [ mkText "25" ]
                                                    ] []
                                                  ]
                                                , mkelem_qn21
                                                  [ mkattr_qn16 [ mkText "26" ]
                                                  ] []
                                                ]
                                              , mkelem_qn21
                                                [ mkattr_qn16 [ mkText "27" ]
                                                ] []
                                              ]
                                            , mkelem_qn21
                                              [ mkattr_qn16 [ mkText "28" ]
                                              ] []
                                            ]
                                          , mkelem_qn21
                                            [ mkattr_qn16 [ mkText "29" ]
                                            ] []
                                          ]
                                        , mkelem_qn21
                                          [ mkattr_qn16 [ mkText "30" ]
                                          ] []
                                        ]
                                      , mkelem_qn21
                                        [ mkattr_qn16 [ mkText "31" ]
                                        ] []
                                      ]
                                    , mkelem_qn21
                                      [ mkattr_qn16 [ mkText "32" ]
                                      ] []
                                    ]
                                  , mkelem_qn21
                                    [ mkattr_qn16 [ mkText "33" ]
                                    ] []
                                  ]
                                , mkelem_qn21
                                  [ mkattr_qn16 [ mkText "34" ]
                                  ] []
                                ]
                              , mkelem_qn21
                                [ mkattr_qn16 [ mkText "35" ]
                                ] []
                              ]
                            , mkelem_qn21
                              [ mkattr_qn16 [ mkText "36" ]
                              ] []
                            ]
                          , mkelem_qn21
                            [ mkattr_qn16 [ mkText "37" ]
                            ] []
                          ]
                        , mkelem_qn21
                          [ mkattr_qn16 [ mkText "38" ]
                          ] []
                        ]
                      , mkelem_qn21
                        [ mkattr_qn16 [ mkText "39" ]
                        ] []
                      ]
                    , mkelem_qn21
                      [ mkattr_qn16 [ mkText "42" ]
                      ] []
                    ]
                  , mkelem_qn21
                    [ mkattr_qn16 [ mkText "43" ]
                    ] []
                  ]
                , mkelem_qn21
                  [ mkattr_qn16 [ mkText "44" ]
                  ] []
                ]
              ]
            ]
          ]
        ]
      ]
    ]
